On Tue, Apr 07, 2020 at 07:12:34PM +0200, Mathieu Othacehe wrote:
> Now it would be nice to support multiple keyboard layout selection from
> the installer, so that a configuration like to one above is generated.
> 
> Thanks,
> 
> Mathieu

Ludo responded to the other bug:

On Tue, Apr 07, 2020 at 11:49:09AM +0200, Ludovic Courtès wrote:
> Ideally we’d also offer a way to choose multiple layouts in the
> installer, so that one can end up with:
>
>   (keyboard-layout "ar,us" #:options '("grp:alt_shift_toggle"))
>

I believe the user should not have to manually select the other layout
(if the user wants they can select via F1 key).  Instead there should
be a sane default (typically U.S. layout).  But since I am used to
Latin script, I may not have good judgment.

IMHO two things should be done:

* Within the installer, a toggleable second keyboard layout should be
  available by default.

  Mathieu enabled changing the layout via the F1 help menu.  This is
  good and more flexible and discoverable, but a key combination is
  more convenient and people appear to be used to Alt+Shift toggling.

* In the config.scm created by the installer, the default
  keyboard-layout should include the second layout.

Both the config.scm default and the installer layout should be the
same, I think.

I attach a proposed solution for the first thing, though it does not
  yet update the toggle option correctly.  If you think the
approach is OK, then I will add more default alternative layouts to
the maybe-add-second-layout procedure.

> Although that’s mostly useful for the console as GDM and GNOME should be
> able to do the right thing.
> 
> Anyway, we can discuss it in a separate issue.
> 
> Ludo’.

The GNOME session after login will ask the user what keyboard layout
they want, but the (I believe changeable) default for GDM comes
from config.scm.

Regards,
Florian
>From 0d3a1ecc214fe55d77f45f4b2e690a93978da9ec Mon Sep 17 00:00:00 2001
From: Florian Pelz <pelzflor...@pelzflorian.de>
Date: Tue, 7 Apr 2020 19:06:31 +0200
Subject: [PATCH] installer: Allow Alt+Shift toggle from non-Latin keyboard
 layouts.

See <https://bugs.gnu.org/40273>.

* gnu/installer/newt/keymap.scm (run-keymap-page): Maybe add second layout.
* gnu/installer/keymap.scm (kmscon-update-keymap): Pass on XKB options.
* gnu/installer/records.scm (<installer>): Adjust code comments.
* gnu/installer.scm (apply-keymap): Pass on XKB options.
(installer-steps): Adjust code comments.
* gnu/packages/patches/kmscon-runtime-keymap-switch.patch: Apply XKB options.
---
 gnu/installer.scm                             | 14 +++++-----
 gnu/installer/keymap.scm                      |  8 ++++--
 gnu/installer/newt/keymap.scm                 | 19 ++++++++++---
 gnu/installer/record.scm                      |  3 ++-
 .../kmscon-runtime-keymap-switch.patch        | 27 ++++++-------------
 5 files changed, 40 insertions(+), 31 deletions(-)

diff --git a/gnu/installer/keymap.scm b/gnu/installer/keymap.scm
index df9fc5e441..d520b782fd 100644
--- a/gnu/installer/keymap.scm
+++ b/gnu/installer/keymap.scm
@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2018 Mathieu Othacehe <m.othac...@gmail.com>
+;;; Copyright © 2020 Florian Pelz <pelzflor...@pelzflorian.de>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -154,8 +155,8 @@ Configuration Database, describing possible XKB 
configurations."
       ((models layouts)
        (values models layouts)))))
 
-(define (kmscon-update-keymap model layout variant)
-  "Update kmscon keymap with the provided MODEL, LAYOUT and VARIANT."
+(define (kmscon-update-keymap model layout variant options)
+  "Update kmscon keymap with the provided MODEL, LAYOUT, VARIANT and OPTIONS."
   (and=>
    (getenv "KEYMAP_UPDATE")
    (lambda (keymap-file)
@@ -175,4 +176,7 @@ Configuration Database, describing possible XKB 
configurations."
          (put-u8 port 0)
 
          (format port variant)
+         (put-u8 port 0)
+
+         (format port options)
          (put-u8 port 0))))))
diff --git a/gnu/installer/newt/keymap.scm b/gnu/installer/newt/keymap.scm
index 0147a0b9d5..ba02085a44 100644
--- a/gnu/installer/newt/keymap.scm
+++ b/gnu/installer/newt/keymap.scm
@@ -1,6 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2018, 2020 Mathieu Othacehe <m.othac...@gmail.com>
 ;;; Copyright © 2019 Ludovic Courtès <l...@gnu.org>
+;;; Copyright © 2020 Florian Pelz <pelzflor...@pelzflorian.de>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -114,8 +115,10 @@ You can switch to different layout at any time from the 
help menu.")))
 
 (define* (run-keymap-page layouts #:key (context #f))
   "Run a page asking the user to select a keyboard layout and variant. LAYOUTS
-is a list of supported X11-KEYMAP-LAYOUT. Return a list of two elements, the
-names of the selected keyboard layout and variant."
+is a list of supported X11-KEYMAP-LAYOUT.  For non-Latin keyboard layouts, a
+second layout and toggle options will be added automatically.  Return a list
+of three elements, the names of the selected keyboard layout, variant and
+optionsxs."
   (define keymap-steps
     (list
      (installer-step
@@ -144,6 +147,15 @@ names of the selected keyboard layout and variant."
                    (gettext (x11-keymap-variant-description variant)
                             "xkeyboard-config"))))))))))
 
+  (define (maybe-add-second-layout first-layout)
+    "Return for a chosen keyboard layout either the same layout or possibly
+the layout plus a suitable second layout, and also variant and options."
+    (match first-layout
+      (("ar" "azerty") (list "ar,fr" "azerty," "grp:alt_shift_toggle"))
+      (("ar" variant) (list "ar,us" variant "grp:alt_shift_toggle"))
+      (("ja" variant) (list "ja,us" variant "grp:alt_shift_toggle"))
+      ((layout variant) (list layout variant ""))))
+
   (define (format-result result)
     (let ((layout (x11-keymap-layout-name
                    (result-step result 'layout)))
@@ -151,7 +163,8 @@ names of the selected keyboard layout and variant."
                           (lambda (variant)
                             (gettext (x11-keymap-variant-name variant)
                                      "xkeyboard-config")))))
-      (list layout (or variant ""))))
+      (maybe-add-second-layout
+       (list layout (or variant "")))))
   (format-result
    (run-installer-steps #:steps keymap-steps)))
 
diff --git a/gnu/installer/record.scm b/gnu/installer/record.scm
index 78acf50c63..13acd33fc6 100644
--- a/gnu/installer/record.scm
+++ b/gnu/installer/record.scm
@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2018, 2020 Mathieu Othacehe <m.othac...@gmail.com>
+;;; Copyright © 2020 Florian Pelz <pelzflor...@pelzflorian.de>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -63,7 +64,7 @@
   (exit-error installer-exit-error)
   ;; procedure void -> void
   (final-page installer-final-page)
-  ;; procedure (layouts context) -> (list layout variant)
+  ;; procedure (layouts context) -> (list layout variant options)
   (keymap-page installer-keymap-page)
   ;; procedure: (#:key supported-locales iso639-languages iso3166-territories)
   ;; -> glibc-locale
diff --git a/gnu/packages/patches/kmscon-runtime-keymap-switch.patch 
b/gnu/packages/patches/kmscon-runtime-keymap-switch.patch
index 656c76fa40..deb5688daf 100644
--- a/gnu/packages/patches/kmscon-runtime-keymap-switch.patch
+++ b/gnu/packages/patches/kmscon-runtime-keymap-switch.patch
@@ -1,14 +1,5 @@
-From 360d44d67e7be46108bec982ff2e79b89f04a9a3 Mon Sep 17 00:00:00 2001
-From: Mathieu Othacehe <m.othac...@gmail.com>
-Date: Thu, 15 Nov 2018 14:34:40 +0900
-Subject: [PATCH] add runtime keymap switch support.
-
----
- src/pty.c                  | 23 ++++++++++-
- src/uterm_input.c          |  2 +
- src/uterm_input_internal.h |  5 +++
- src/uterm_input_uxkb.c     | 83 ++++++++++++++++++++++++++++++++++++++
- 4 files changed, 111 insertions(+), 2 deletions(-)
+By Mathieu Othacehe <m.othac...@gmail.com>.
+Modified by Florian Pelz <pelzflor...@pelzflorian.de>.
 
 diff --git a/src/pty.c b/src/pty.c
 index 1443f4a..f64cb5b 100644
@@ -124,7 +115,7 @@ index 04e6cc9..ec44459 100644
                     uint16_t key_state,
                     uint16_t code);
 diff --git a/src/uterm_input_uxkb.c b/src/uterm_input_uxkb.c
-index 925c755..4760972 100644
+index 925c755..5d5c22e 100644
 --- a/src/uterm_input_uxkb.c
 +++ b/src/uterm_input_uxkb.c
 @@ -31,6 +31,9 @@
@@ -137,7 +128,7 @@ index 925c755..4760972 100644
  #include <xkbcommon/xkbcommon.h>
  #include "shl_hook.h"
  #include "shl_llog.h"
-@@ -178,6 +181,86 @@ static void timer_event(struct ev_timer *timer, uint64_t 
num, void *data)
+@@ -178,6 +181,87 @@ static void timer_event(struct ev_timer *timer, uint64_t 
num, void *data)
        shl_hook_call(dev->input->hook, dev->input, &dev->repeat_event);
  }
  
@@ -145,11 +136,11 @@ index 925c755..4760972 100644
 +{
 +      struct uterm_input_dev *dev = data;
 +      char in;
-+      char keymap[3][255];
++      char keymap[4][255];
 +      int pos = 0;
 +      int curr_keymap = 0;
 +      int ret;
-+      char *model, *layout, *variant;
++      char *model, *layout, *variant, *options;
 +
 +      if (!(mask & EV_READABLE))
 +              return;
@@ -159,6 +150,7 @@ index 925c755..4760972 100644
 +      model = keymap[0];
 +      layout = keymap[1];
 +      variant = keymap[2];
++      options = keymap[3];
 +
 +      do {
 +        ret = read(dev->rupdate_fd, &in, sizeof(in));
@@ -175,7 +167,7 @@ index 925c755..4760972 100644
 +
 +      llog_info(dev->input, "HANDLER CALLED %s|%s|%s\n",
 +                model, layout, variant);
-+      uxkb_desc_init(dev->input, model, layout, variant, NULL, NULL);
++      uxkb_desc_init(dev->input, model, layout, variant, options, NULL);
 +
 +      dev->state = xkb_state_new(dev->input->keymap);
 +      if (!dev->state) {
@@ -224,6 +216,3 @@ index 925c755..4760972 100644
  int uxkb_dev_init(struct uterm_input_dev *dev)
  {
        int ret;
--- 
-2.17.1
-
diff --git a/gnu/installer.scm b/gnu/installer.scm
index 75c5a01570..72b7d195fd 100644
--- a/gnu/installer.scm
+++ b/gnu/installer.scm
@@ -2,6 +2,7 @@
 ;;; Copyright © 2018, 2020 Mathieu Othacehe <m.othac...@gmail.com>
 ;;; Copyright © 2019, 2020 Ludovic Courtès <l...@gnu.org>
 ;;; Copyright © 2019, 2020 Tobias Geerinckx-Rice <m...@tobias.gr>
+;;; Copyright © 2020 Florian Pelz <pelzflor...@pelzflorian.de>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -170,9 +171,9 @@ been performed at build time."
 (define apply-keymap
   ;; Apply the specified keymap. Use the default keyboard model.
   #~(match-lambda
-      ((layout variant)
+      ((layout variant options)
        (kmscon-update-keymap (default-keyboard-model)
-                             layout variant))))
+                             layout variant options))))
 
 (define* (compute-keymap-step context)
   "Return a gexp that runs the keymap-page of INSTALLER and install the
@@ -235,12 +236,13 @@ selected keymap."
 
          ;; The installer runs in a kmscon virtual terminal where loadkeys
          ;; won't work. kmscon uses libxkbcommon as a backend for keyboard
          ;; input. It is possible to update kmscon current keymap by sending it
          ;; a keyboard model, layout and variant, in a somehow similar way as
          ;; what is done with setxkbmap utility.
          ;;
          ;; So ask for a keyboard model, layout and variant to update the
-         ;; current kmscon keymap.
+         ;; current kmscon keymap.  For non-Latin layouts, we add an
+         ;; appropriate second layout and toggle via Alt+Shift.
          (installer-step
           (id 'keymap)
           (description (G_ "Keyboard mapping selection"))
@@ -235,12 +236,13 @@ selected keymap."
          ;; The installer runs in a kmscon virtual terminal where loadkeys
          ;; won't work. kmscon uses libxkbcommon as a backend for keyboard
-         ;; input. It is possible to update kmscon current keymap by sending it
-         ;; a keyboard model, layout and variant, in a somehow similar way as
-         ;; what is done with setxkbmap utility.
+         ;; input. It is possible to update kmscon current keymap by sending
+         ;; it a keyboard model, layout, variant and options, in a somehow
+         ;; similar way as what is done with setxkbmap utility.
          ;;
          ;; So ask for a keyboard model, layout and variant to update the
-- 
2.26.0

Reply via email to